# coding=utf-8

"""Dublin Core Metadata Initiative, see http://dublincore.org/documents/dcmi-terms/"""

from epubaker.metas.attrs import Attrs, AltScript, Dir, FileAs, Id, Role, Lang
from epubaker.xl import Element, URI_XML


def always_true(*args, **kwargs):
    pass

l = [
    'abstract', 'accessRights', 'accrualMethod', 'accrualPeriodicity', 'accrualPolicy', 'alternative', 'audience',
    'available',
    'bibliographicCitation',
    'conformsTo', 'contributor', 'coverage', 'created', 'creator',
    'date', 'dateAccepted', 'dateCopyrighted', 'dateSubmitted', 'description',
    'educationLevel', 'extent',
    'format',
    'hasFormat', 'hasPart', 'hasVersion',
    'identifier', 'instructionalMethod',
    'isFormatOf', 'isPartOf', 'isReferencedBy', 'isReplacedBy', 'isRequiredBy', 'issued', 'isVersionOf',
    'language', 'license',
    'mediator', 'medium', 'modified',
    'provenance', 'publisher',
    'references', 'relation', 'replaces', 'requires', 'rights', 'rightsHolder',
    'source', 'spatial', 'subject',
    'tableOfContents', 'temporal', 'title', 'type',
    'valid'
]


check_funcs = {
    'modified': always_true,
}


check_funcs.update(dict((one, always_true) for one in l if one not in check_funcs.keys()))


_attr_check_funcs = {
    # identifier only
    'opf:scheme': always_true,

    'opf:alt-script': always_true,
    'dir': always_true,
    'opf:file-as': always_true,
    'id': always_true,
    'opf:role': always_true,
    'xml:lang': always_true,

    # subject only
    'opf:authority': always_true,

    # Meta only
    'scheme': always_true
}

URI_DC = 'http://purl.org/dc/elements/1.1/'
URI_OPF = 'http://www.idpf.org/2007/opf'

namespace_map = {
    'dc': URI_DC,
    'opf': URI_OPF,
    'xml': URI_XML
}


class _Base(Attrs):
    def __init__(self, text):
        check_funcs[self.__class__.__name__](text)
        Attrs.__init__(self)

        self._text = text

    @property
    def text(self):
        return self._text

    def to_element(self):
        e = Element((None, 'meta'))

        e.attributes[(None, 'property')] = 'dcterms:{}'.format(self.__class__.__name__)

        for attr_name, value in self._attrs.items():

            uri = None
            if ':' in attr_name:
                prefix, attr = attr_name.split(':')
                uri = namespace_map[prefix]
                e.prefixes[uri] = prefix

            else:
                attr = attr_name

            e.attributes[(uri, attr)] = value

        e.children.append(self.text)

        return e


_classes = {}

for k, v in check_funcs.items():

    _classes[k] = type(k, (_Base, AltScript, Dir, FileAs, Id, Role, Lang), {})


def get_dcterm(name):
    """get a term class by term name"""
    return _classes[name]




# coding=utf-8

"""
Dublin Core Metadata Element Set, see http://dublincore.org/documents/dces/

All classes in this module
"""

import uuid

from epubaker.xl import Element, URI_XML

from .attrs import Id, Scheme, AltScript, Dir, FileAs, Role, Lang, Authority, Attrs

URI_DC = 'http://purl.org/dc/elements/1.1/'
URI_OPF = 'http://www.idpf.org/2007/opf'

namespace_map = {
    'opf': URI_OPF,
    'xml': URI_XML
}


class Base(Attrs):
    """the name"""
    def __init__(self, text):
        Attrs.__init__(self)

        self._text = text

    @property
    def text(self):
        return self._text

    def to_element(self):
        e = Element((URI_DC, self.__class__.__name__.lower()))
        e.prefixes[URI_DC] = 'dc'

        for attr_name, value in self._attrs.items():

            if ':' in attr_name:
                prefix, attr = attr_name.split(':')
                uri = namespace_map[prefix]
                e.prefixes[uri] = prefix

                e.attributes[(uri, attr)] = value

            else:
                attr = attr_name

                e.attributes[(None, attr)] = value

        e.children.append(self._text)

        return e


##########################################################################

class Identifier(Base, Id, Scheme):
    """identifier"""
    def __init__(self, text):
        Base.__init__(self, text)
        self.id = 'id_' + uuid.uuid4().hex


class Title(Base, AltScript, Dir, FileAs, Id, Lang):
    """title of Book"""


class Language(Base, Id):
    """https://tools.ietf.org/html/rfc5646

    example: en-US
    """


class Contributor(Base, AltScript, Dir, FileAs, Id, Role, Lang):
    """contributor"""


class Coverage(Base, Dir, Lang):
    """coverage"""


class Creator(Base, AltScript, Dir, FileAs, Id, Role, Lang):
    """creator"""


class Date(Base, Id):
    """date"""


class Description(Base, Dir, Id, Lang):
    """description"""


class Format(Base, Id):
    """format"""


class Publisher(Base, AltScript, Dir, FileAs, Id, Lang):
    """publisher"""


class Relation(Base, Dir, Id, Lang):
    """relation"""


class Rights(Base, Dir, Id, Lang):
    """rights"""


class Source(Base, Id):
    """source"""


class Subject(Base, Dir, Id, Lang, Authority):
    """subject"""


class Type(Base, Id):
    """type"""



# coding=utf-8

import os
import zipfile


from epubaker.epub import Epub, File, OPF_NS, ROOT_OF_OPF, CONTAINER_PATH

from epubaker.metas.dcmes import URI_DC

from epubaker.metas.epub2_meta import Cover

from epubaker.xl import Xl, Element, pretty_insert


class Epub2(Epub):
    def __init__(self):
        Epub.__init__(self)

    def _make_metadata_element(self):
        """
        :return: Metadata Element
        :rtype: Element
        """
        metadata = Element('metadata', prefixes={URI_DC: 'dc'})

        for m in self.metadata:
            if isinstance(m, Cover):
                cover = Element('meta', attributes={'name': 'cover', 'content': self._find_id(m.filepath)})
                metadata.children.append(cover)
            else:
                metadata.children.append(m.to_element())

        return metadata

    def _get_opf_xmlstring(self):

        package = Element('package', prefixes={OPF_NS: None}, attributes={'version': '2.0'})

        # unique - identifier = "pub-id"
        if self._find_unique_id():
            package.attributes['unique-identifier'] = self._find_unique_id()

        # Metadata
        package.children.append(self._make_metadata_element())

        # Manifest
        manifest = self._make_manifest_element()
        package.children.append(manifest)

        # Find ncx id for spine
        toc_ncx_item_e_id = self._find_ncx_id(manifest.children)

        # Spine
        spine = self._make_spine_element()
        package.children.append(spine)
        spine.attributes['toc'] = toc_ncx_item_e_id

        return Xl(root=pretty_insert(package, dont_do_when_one_child=True)).string()

    def write(self, filename):

        # put ncx to temp files
        ncx_xmlstring = pretty_insert(self._make_ncx_element(), dont_do_when_one_child=True).string()
        toc_ncx_filename = self._get_unused_filename(None, 'toc.ncx')
        self._temp_files[toc_ncx_filename] = File(ncx_xmlstring.encode(), mime='application/x-dtbncx+xml')

        # get opf name & data
        opf_data = self._get_opf_xmlstring().encode()
        opf_filename = self._get_unused_filename(None, 'package.opf')

        # get container data
        container_data = self._get_container_xmlstring(ROOT_OF_OPF + '/' + opf_filename).encode()

        # make zip file
        z = zipfile.ZipFile(filename, 'w', compression=zipfile.ZIP_DEFLATED)

        # write mimetype as first file in zip
        z.writestr('mimetype', 'application/epub+zip'.encode('ascii'), compress_type=zipfile.ZIP_STORED)

        # wirte custom files
        for filename, fil in self.files.items():
            z.writestr(ROOT_OF_OPF + os.sep + filename, fil.binary, zipfile.ZIP_DEFLATED)

        # write temp files
        for filename, fil in self._temp_files.items():
            z.writestr(ROOT_OF_OPF + os.sep + filename, fil.binary, zipfile.ZIP_DEFLATED)

        # write opf data
        z.writestr(ROOT_OF_OPF + '/' + opf_filename, opf_data, zipfile.ZIP_DEFLATED)

        # write container
        z.writestr(CONTAINER_PATH, container_data, zipfile.ZIP_DEFLATED)

        z.close()

        self._temp_files.clear()

    write.__doc__ = Epub.write.__doc__



# coding=utf-8

from .attrs import Attrs, AltScript, Dir, FileAs, Id, Scheme, Lang
from .dcmes import Base


class Property(Attrs):
    @property
    def property(self):
        """xml attribute: `property`"""
        return self._attrs.setdefault('property')

    @property.setter
    def property(self, value):
        self._attrs['property'] = value


class Meta3(Base, AltScript, Dir, FileAs, Id, Property, Scheme, Lang):
    """meta for Epub3.metadata"""
    def __init__(self, property_, text):
        Base.__init__(self, text)

        self.property = property_



# coding=utf-8

from __future__ import print_function

from abc import abstractmethod

import copy

from hooky import List, Dict

import xml.parsers.expat


class HandlerError(Exception):
    pass


def parse(xmlstr, debug=False):
    """
    parse XML string to Xl object

    :param xmlstr:
    :type xmlstr: str
    :param debug:
    :type debug: bool
    :return: object of :class:`Xl`
    :rtype: Xl
    """

    xl = Xl()

    out_element = [None]

    elements = []

    ns_list = []

    s = ['']

    def start_element(name, attrs):
        _do_string()

        print('start_element', name) if debug else None

        attributes = {}

        l = name.rsplit(' ', 1)
        tag = l[-1]

        uri = None
        if len(l) > 1:
            uri = l[0]

        for key, value in attrs.items():
            l = key.rsplit(' ', 1)

            attr_name = l[-1]

            attr_uri = None
            if len(l) > 1:
                attr_uri = l[0]

            attributes[(attr_uri, attr_name)] = value

        prefixes = {}

        for _uri, _prefix in ns_list:

            prefixes[_uri] = _prefix

        e = Element(tag=(uri, tag), attributes=attributes, prefixes=prefixes)

        if elements:
            elements[-1].children.append(e)

        elements.append(e)

        # nonlocal out_element
        if not out_element[0]:
            out_element[0] = e

    def end_element(name):
        _do_string()
        print('end_element', name) if debug else None
        elements.pop()

    def start_namespace(prefix, uri):
        print('start_namespace') if debug else None
        ns_list.append((uri, prefix))

    def end_namespace(prefix):
        print('end_namespace') if debug else None
        ns_list.pop()

    def character_data_handler(data):
        print('Character html: "{}"'.format(data)) if debug else None
        # nonlocal s
        s[0] += data

    def _do_string():
        # nonlocal s
        if s[0] and elements:
            string = s[0]
            elements[-1].children.append(string)
            s[0] = ''

    def start_doc_type_decl(doc_type_name, system_id, public_id, has_internal_subset):
        xl.doc_type = DocType(doc_type_name=doc_type_name, system_id=system_id, public_id=public_id)

        if has_internal_subset == 1:
            raise HandlerError('Has internal subset, cannot handler it!')

    def decl_handler(version, encoding, standalone):
        standalone_ = None

        if standalone == 1:
            standalone_ = True
        elif standalone == 0:
            standalone_ = False
        elif standalone == -1:
            standalone_ = None

        xl.header = Header(version=version, encoding=encoding, standalone=standalone_)

    p = xml.parsers.expat.ParserCreate(namespace_separator=' ', encoding='UTF=8')

    p.XmlDeclHandler = decl_handler

    p.StartDoctypeDeclHandler = start_doc_type_decl

    # internal dtd
    # p.EntityDeclHandler = entity_decl_handler

    p.StartElementHandler = start_element

    p.EndElementHandler = end_element

    p.StartNamespaceDeclHandler = start_namespace

    p.EndNamespaceDeclHandler = end_namespace

    p.CharacterDataHandler = character_data_handler

    p.Parse(xmlstr, 1)

    xl.root = out_element[0]

    return xl


class ObjectAttributeError(Exception):
    pass


def _name_check(name):
    pass


def _nsmap_check(nsmap):
    pass


def _nsuri_check(uri, namespaces):
    if uri is None:
        pass
    elif uri not in namespaces.keys():
        raise Exception


def clean_whitespaces(element):
    """
    :param element:
    :return: A copy of the element, all whitespace characters have been stripped from the beginning and the end of the
     text node in the children and children's children and so on. delete the text node If it is empty.
    """
    if not isinstance(element, Element):
        raise TypeError

    new_element = Element(tag=copy.deepcopy(element.tag),
                          attributes=copy.deepcopy(element.attributes),
                          prefixes=copy.deepcopy(element.prefixes))

    for child in element.children:
        if isinstance(child, str):
            new_text = child.strip()

            if new_text:
                new_element.children.append(new_text)

        elif isinstance(child, Element):
            new_element.children.append(clean_whitespaces(child))

    return new_element


def _is_straight_line(element):
    if len(element.children) == 0:
        return True

    if len(element.children) == 1:
        if isinstance(element.children[0], Element):
            return _is_straight_line(element.children[0])
        else:
            return True

    elif len(element.children) > 1:
        return False


def pretty_insert(element, start_indent=0, step=4, dont_do_when_one_child=True):
    """
    Modify the copy of the element, to make it looks more pretty and clear.

    :param element:
    :type element: Element
    :param start_indent:
    :type start_indent: int
    :param step:
    :type step: int
    :param dont_do_when_one_child:
    :type dont_do_when_one_child: bool
    :return: object of :class:`Element`
    """

    new_element = Element(tag=copy.deepcopy(element.tag),
                          attributes=copy.deepcopy(element.attributes),
                          prefixes=copy.deepcopy(element.prefixes))

    _indent_text = '\n' + ' ' * (start_indent + step)

    if _is_straight_line(element) and dont_do_when_one_child:
        new_element.children = copy.deepcopy(element.children)

    elif element.children:
        for child in element.children:

            if isinstance(child, str):
                new_text = _indent_text + child

                new_element.children.append(new_text)

            elif isinstance(child, Element):
                new_element.children.append(_indent_text)

                new_element.children.append(pretty_insert(element=child,
                                                          start_indent=start_indent + step,
                                                          step=step,
                                                          dont_do_when_one_child=dont_do_when_one_child,
                                                          ))

        new_element.children.append('\n' + ' ' * start_indent)

    return new_element


URI_XML = 'http://www.w3.org/XML/1998/namespace'


class XLError(Exception):
    pass


class Xl(object):
    def __init__(self, header=None, doc_type=None, root=None):
        """
        :param header:
        :type header: Header
        :param doc_type:
        :type doc_type: DocType
        :param root:
        :type root: Element
        """

        self.header = header or header
        """object of :class:`Header`"""

        self.doc_type = doc_type
        """object of :class:`DocType`"""

        self.root = root
        """object of :class:`Element`"""

    def string(self):
        """To xml string"""
        s = ''

        if self.header:
            s += self.header.string() + '\n'

        if self.doc_type:
            s += self.doc_type.string() + '\n'

        s += self.root.string()

        return s


class _Node(object):
    @abstractmethod
    def string(self):
        pass


class Header(_Node):
    """
    Handle XML header node
    """
    def __init__(self, version=None, encoding=None, standalone=None):
        """

        :param version:
        :type version: str
        :param encoding:
        :type encoding: str
        :param standalone:
        :type standalone: bool
        """
        self.version = version or '1.0'
        self.encoding = encoding or 'utf-8'
        self.standalone = standalone

    def string(self):
        if not (self.version or self.encoding or self.standalone):
            return ''

        s = ''

        s += '<?xml'

        if self.version:
            s += ' version="{}"'.format(self.version)

        if self.encoding:
            s += ' encoding="{}"'.format(self.encoding)

        if self.standalone is not None:
            s += ' standalone="{}"'.format('yes' if self.standalone else 'no')

        s += ' ?>'

        return s


class DocType(_Node):
    """
    Handle XML doc type node
    """
    def __init__(self, doc_type_name, system_id, public_id):
        self.doc_type_name = doc_type_name
        self.system_id = system_id
        self.public_id = public_id

    def string(self):
        s = ''
        s += '<!DOCTYPE'

        s += ' {}'.format(self.doc_type_name)
        s += ' "{}"'.format(self.public_id)
        s += ' "{}"'.format(self.system_id)

        s += '>'

        return s


class Element(_Node):
    """
    Handle XML element node.
    """
    def __init__(self, tag=None, attributes=None, prefixes=None):
        _Node.__init__(self)
        """
        :param tag:
        :type tag: tuple or str
        :param attributes:
        :type attributes: _Attributes or dict
        :param prefixes:
        :type prefixes: _Prefixes or dict
        """

        self._tag = None

        self.tag = tag
        """tuple object of length 2.

        First in the tuple is the url of the namespaces,
        the second is the xml element tag you know ordinarily.
        """

        self._attributes = _Attributes(attributes) if attributes else _Attributes()
        """dict-like.

        Store xml attribute names and values in *keys* and *values*

        """

        self._prefixes = _Prefixes(prefixes) if prefixes else _Prefixes()
        """dict-like.

        Store xml namespaces urls and prefixes in *keys* and *values*

        Ignore this is fine, because you will get automatic prefixes for the namespaces.
        """

        self._children = _Children()
        """list-like.

        Store children Node"""

    @property
    def tag(self):
        return self._tag

    @tag.setter
    def tag(self, value):
        if not isinstance(value, tuple):
            value = (None, value)

        if len(value) != 2:
            raise ValueError

        if value[0] is not None:
            if isinstance(value[0], str):
                pass
            elif isinstance(value[0], unicode):
                pass
            else:
                raise ValueError

        if isinstance(value[1], str):
            pass
        elif isinstance(value[1], unicode):
            pass
        else:
            raise ValueError

        self._tag = value

    @property
    def prefixes(self):
        return self._prefixes

    @prefixes.setter
    def prefixes(self, value):
        self._prefixes = value

    @property
    def attributes(self):
        return self._attributes

    @attributes.setter
    def attributes(self, value):
        self._attributes = value

    @property
    def children(self):
        return self._children

    @children.setter
    def children(self, value):
        self._children = value
        self.__dict__['children'] = value

    def string(self, inherited_prefixes=None):
        """to string, you may want to see :class:`Xl.string`"""
        if self.tag[1] is None:
            raise TypeError

        inherited_prefixes = inherited_prefixes or _Prefixes()

        auto_prefixs = _Prefixes()

        def make_a_auto_prefix(_uri):
            _prefix_num = 0
            while 'prefix' + str(_prefix_num) in \
                    [one for one in self.prefixes.values()] + \
                    [one for one in inherited_prefixes.values()] + \
                    [one for one in auto_prefixs.values()]:

                _prefix_num += 1

            _prefix = 'prefix' + str(_prefix_num)
            auto_prefixs[_uri] = _prefix

            return _prefix

        def get_prefix(_uri):

            if _uri in self.prefixes.keys():
                _prefix = self.prefixes[_uri]

            elif _uri in inherited_prefixes.keys():
                _prefix = inherited_prefixes[_uri]

            elif _uri in auto_prefixs.keys():
                _prefix = auto_prefixs[_uri]

            else:
                raise ValueError

            return _prefix

        s = '<'

        ################################################################################################################
        # processing xml tag
        if self.tag[0] is not None:
            try:
                prefix = get_prefix(self.tag[0])
            except ValueError:
                prefix = make_a_auto_prefix(self.tag[0])

            if prefix is not None:
                full_name = '{}:{}'.format(prefix, self.tag[1])
            else:
                full_name = self.tag[1]
        else:
            full_name = self.tag[1]

        s += full_name

        ################################################################################################################
        # processing xml attributes
        _attrs_string_list = []
        for attr_name, attr_value in self.attributes.items():

            if attr_name[0] is not None:
                try:
                    prefix = get_prefix(attr_name[0])
                except ValueError:
                    prefix = make_a_auto_prefix(attr_name[0])
                _attrs_string_list.append('{}:{}="{}"'.format(prefix, attr_name[1], attr_value))

            else:
                _attrs_string_list.append('{}="{}"'.format(attr_name[1], attr_value))

        if _attrs_string_list:
            s += ' '
            s += ' '.join(_attrs_string_list)

        ################################################################################################################
        # processing xml prefixes
        _prefix_string_list = []
        for url, prefix in self.prefixes.items():
            if url == URI_XML:
                continue

            if url in inherited_prefixes.keys() and prefix == inherited_prefixes[url]:
                continue

            if url:
                if prefix:
                    _prefix_string_list.append('xmlns:{}="{}"'.format(prefix, url))
                else:
                    _prefix_string_list.append('xmlns="{}"'.format(url))

        if _prefix_string_list:
            s += ' '
            s += ' '.join(_prefix_string_list)

        ################################################################################################################
        # processing children
        if self.children:
            s += '>'

            prefixes_for_subs = inherited_prefixes.copy()
            prefixes_for_subs.update(self.prefixes)
            prefixes_for_subs.update(auto_prefixs)

            for child in self.children:
                if isinstance(child, Element):
                    s += child.string(inherited_prefixes=prefixes_for_subs)

                # elif isinstance(child, Text):
                #    s += child.string()

                elif isinstance(child, str):
                    s += _escape(child)

            s += '</{}>'.format(full_name)

        else:
            s += ' />'

        return s


class _Prefixes(Dict):
    def _before_add(self, key=None, item=None):
        if key == URI_XML and item != 'xml':
            raise ValueError

        if item in self.values() and (key not in self.keys() or item != self[key]):
            raise ValueError

    def __init__(self, prefixes=None):
        Dict.__init__(self)

        self[URI_XML] = 'xml'

        if prefixes:
            self.update(prefixes)


class _Attributes(Dict):
    def __init__(self, attributes=None):
        Dict.__init__(self)

        if attributes:
            self.update(attributes)

    def __setitem__(self, key, value):
        if not isinstance(key, tuple):
            key = (None, key)

        if key[0] is not None:
            if isinstance(key[0], str):
                pass

            elif isinstance(key[0], unicode):
                pass

            else:
                raise KeyError

        if isinstance(key[1], str):
            pass

        elif isinstance(key[1], unicode):
            pass

        else:
            raise KeyError

        if key == (None, 'xmlns'):
            raise AttributeError

        Dict.__setitem__(self, key, value)


class _Children(List):
    def __init__(self):
        List.__init__(self)

    def _before_add(self, key=None, item=None):
        if isinstance(item, (_Node, str)):
            pass

        elif isinstance(item, unicode):
            pass

        else:
            raise TypeError('{} is not legal'.format(item.__class__.__name__))


def _escape(string):
    """

    :param string:
     :type string: str
    :return:
     :rtype: str
    """
    s = ''
    for char in string:
        if char == '&':
            s += '&amp;'
        elif char == '<':
            s += '&lt;'
        elif char == '>':
            s += '&gt;'
        else:
            s += str(char)
    return s



# coding=utf-8

# Image Types
GIF = 'image/gif'  # ['.gif'], 'Images'],
JPEG = 'image/jpeg'  # ['.jpg', 'jpeg'], 'Images'],
PNG = 'image/png'  # ['.png'], 'Images'],
SVG = 'image/svg+xml'  # ['.svg'], 'Images'],

# Application Types
XHTML = 'application/xhtml+xml'  # ['.html', '.xhtml'], 'Text'],
FONT_SFNT = 'application/font-sfnt'  # ['.otf', '.ttf', '.ttc'], 'Fonts'],  # old 'application/vnd.ms-opentype'
FONT_WOFF = 'application/font-woff'  # ['.woff'], 'Fonts'],
SMIL = 'application/smil+xml'  # [], 'Text'],  # EPUB Media Overlay documents
PLS = 'application/pls+xml'  # [], ''],  # Text-to-Speech (TTS) Pronunciation lexicons

# ncx
NCX = 'application/x-dtbncx+xml'  # [], ''],

# Audio Types
MP3 = 'audio/mpeg'  # ['.mp3'], ''],
AAC = 'audio/mp4'  # ['.mp4'], ''],

# Text Types
HTML = 'text/html'  # [], 'Text'],
CSS = 'text/css'  # ['.css'], 'Styles'],
JS = 'text/javascript'  # ['.js'], 'Scripts'],

# Font Types
FONT_WOFF2 = 'font/woff2'  # ['.woff2'], 'Fonts'],


map_from_extension = {
    '.gif': GIF,
    '.jpg''.jpeg': JPEG, '.jpeg': JPEG,
    '.png': PNG,
    '.svg': SVG,

    '.xhtml': XHTML,
    '.otf': FONT_SFNT, '.ttf': FONT_SFNT, '.ttc': FONT_SFNT,
    '.woff': FONT_WOFF,
    '.smil': SMIL, '.smi': SMIL,
    '.pls': PLS,

    '.ncx': NCX,

    '.mp3': MP3,
    '.aac': AAC,

    '.html': HTML,
    '.css': CSS,
    '.js': JS,

    '.woff2': FONT_WOFF2
}



# coding=utf-8


class Attrs(object):
    def __init__(self):
        self._attrs = {}


class Id(Attrs):
    @property
    def id(self):
        """xml attributie: `id`
        """
        return self._attrs.setdefault('id')

    @id.setter
    def id(self, value):
        self._attrs['id'] = value


class Scheme(Attrs):
    @property
    def scheme(self):
        """xml attribute: `opf:scheme`"""
        return self._attrs.setdefault('opf:scheme')

    @scheme.setter
    def scheme(self, value):
        self._attrs['opf:scheme'] = value


class AltScript(Attrs):
    @property
    def alt_script(self):
        """xml attribute: `opf:alt-script`"""
        return self._attrs.setdefault('opf:alt-script')

    @alt_script.setter
    def alt_script(self, value):
        self._attrs['opf:alt-script'] = value


class Dir(Attrs):
    @property
    def dir(self):
        """
        "ltr" (left-to-right) or "rtl" (right-to-left)

        xml attribute: `dir`
        """
        return self._attrs.setdefault('dir')

    @dir.setter
    def dir(self, value):
        if value not in ('ltr', 'rtl'):
            raise ValueError('must be "ltr" or "rtl"')

        self._attrs['dir'] = value


class FileAs(Attrs):
    @property
    def file_as(self):
        """xml attribute: `opf:file-as`"""
        return self._attrs.setdefault('opf:file-as')

    @file_as.setter
    def file_as(self, value):
        self._attrs['opf:file-as'] = value


class Role(Attrs):
    @property
    def role(self):
        """xml attribute: `opf:role`"""
        return self._attrs.setdefault('opf:role')

    @role.setter
    def role(self, value):
        self._attrs['opf:role'] = value


class Lang(Attrs):
    @property
    def lang(self):
        """xml attribute: `xml:lang`
        """
        return self._attrs.setdefault('xml:lang')

    @lang.setter
    def lang(self, value):
        self._attrs['xml:lang'] = value


class Authority(Attrs):
    @property
    def authority(self):
        """xml attribute: `authority`"""
        return self._attrs.setdefault('authority')

    @authority.setter
    def authority(self, value):
        self._attrs['authority'] = value



# coding=utf-8

import string
import uuid

import io
import os
import random
from PIL import Image
from abc import abstractmethod
from hooky import List, Dict


import epubaker.version
from epubaker import mimes
from epubaker.metas import Identifier
from epubaker.tools import relative_path
from epubaker.xl import Xl, Element, pretty_insert


CONTAINER_PATH = 'META-INF' + os.sep + 'container.xml'
ROOT_OF_OPF = 'EPUB'

OPF_NS = 'http://www.idpf.org/2007/opf'


class Metadata(List):
    """list-like.

    Store metadata, such as author, publisher etc.

    see :mod:`epubaker.metas`"""

    pass


########################################################################################################################
# Files File
########################################################################################################################
class Files(Dict):
    """dict-like.

    Store file path and :class:`File` objects from `key` and `item`.
    Any file you want to package them into the book, you have to use this."""
    def _before_add(self, key=None, item=None):
        if not isinstance(item, File):
            raise TypeError


class File(object):
    def __init__(self, binary, mime=None, fallback=None):
        """
        :param binary: binary data
        :type binary: bytes
        :param mime: mime
        :type mime: str
        :param fallback: file path
        :type fallback: str
        """

        self._binary = binary
        self.mime = mime
        # self.identification = identification or 'id_' + uuid.uuid4().hex
        self.fallback = fallback

    @property
    def binary(self):
        """as class parmeter"""
        return self._binary


########################################################################################################################
# Spine Joint
########################################################################################################################
class Spine(List):
    """list-like.

    "The spine defines the default reading order"

    store :class:`Joint` objects.
    """

    def _before_add(self, key=None, item=None):
        if not isinstance(item, Joint):
            raise TypeError


class Joint:
    def __init__(self, path, linear=None):
        """
        :param path: file path, in Epub.Files.keys()
        :type path: str
        :param linear: I don't know what is this mean. visit http://idpf.org to figure out by yourself.
        :type linear: bool
        """
        self._path = path
        self.linear = linear

    @property
    def path(self):
        """as class parmeter"""
        return self._path


########################################################################################################################
# TOC Section
########################################################################################################################
class Toc(List):
    """list-like.

    table of contents

    store :class:`Section` objects.
    """

    def __init__(self):
        List.__init__(self)

        self.title = 'Table of Contents'

        self.ncx_depth = -1
        self.ncx_totalPageCount = -1
        self.ncx_maxPageNumber = -1

    def _before_add(self, key=None, item=None):
        if not isinstance(item, Section):
            raise TypeError


class _SubSections(List):
    def _before_add(self, key=None, item=None):
        if not isinstance(item, Section):
            raise TypeError


class Section(object):
    """
    Store title, href and sub :class:`Section` objects.
    """

    def __init__(self, title, href=None):
        """
        :param title: title of content.
        :type title: str
        :param href: html link to a file path in :class:`Epub.files`, can have a bookmark. example: `text/a.html#hello`
        :type href: str
        """
        self._title = title
        self._href = href
        self._subs = _SubSections()
        self._hidden_subs = None

    @property
    def title(self):
        """as class parameter"""
        return self._title

    @title.setter
    def title(self, value):
        self._title = value

    @property
    def href(self):
        """as class parmeter"""
        return self._href

    @href.setter
    def href(self, value):
        self._href = value

    @property
    def subs(self):
        """list-like, store sub :class:`Section` objects."""
        return self._subs

    @property
    def hidden_subs(self):
        """bool: True for fold sub sections, False unfold.

        some reader just don't show sub sections when this is True,

        but I think it's mean FOLD sub sections and you can unfold it to show subs.

        this i for epub3 nav only."""
        return self._hidden_subs

    @hidden_subs.setter
    def hidden_subs(self, value):
        if value not in (True, False):
            raise ValueError
        else:
            self._hidden_subs = value


########################################################################################################################
# Epub
########################################################################################################################
class Epub(object):
    def __init__(self):
        self._metadata = Metadata()
        setattr(self._metadata, '_epub', self)

        self._files = Files()
        setattr(self._files, '_epub', self)

        self._spine = Spine()
        setattr(self._spine, '_epub', self)

        # for opf etc.
        self._temp_files = Files()
        setattr(self._temp_files, '_epub', self)

        self._toc = Toc()

    metadata = property(lambda self: self._metadata, doc=str(Metadata.__doc__ if Metadata.__doc__ else ''))

    files = property(lambda self: self._files, doc=str(Files.__doc__ if Files.__doc__ else ''))

    spine = property(lambda self: self._spine, doc=str(Spine.__doc__ if Spine.__doc__ else ''))

    toc = property(lambda self: self._toc, doc=str(Toc.__doc__ if Toc.__doc__ else ''))

    @staticmethod
    def _find_ncx_id(items):
        ncx_id = None
        for item in items:
            if item.attributes[(None, 'media-type')] == mimes.NCX:
                ncx_id = item.attributes[(None, 'id')]
                break
        return ncx_id

    def _find_unique_id(self):
        """
        :return:
         :rtype: str or none
        """
        for m in self.metadata:
            if isinstance(m, Identifier):
                return m.to_element().attributes[(None, 'id')]
        return None

    def _find_id(self, filepath):
        for item in self._make_manifest_element().children:
            if item.attributes[(None, 'href')] == filepath:
                return item.attributes[(None, 'id')]
        return None

    def _make_ncx_element(self):

        ncx = Element('ncx',
                      attributes={'version': '2005-1'},
                      prefixes={'http://www.daisy.org/z3986/2005/ncx/': None})
        head = Element('head')
        ncx.children.append(head)

        # same as dc:Identifier

        identifier_text = None
        for m in self.metadata:
            if isinstance(m, Identifier):
                identifier_text = m.text
                break

        head.children.append(Element('meta', attributes={'name': 'dtb:uid', 'content': identifier_text}))

        head.children.append(Element('meta', attributes={'name': 'dtb:depth', 'content': self.toc.ncx_depth}))

        head.children.append(Element('meta', attributes={'name': 'dtb:totalPageCount',
                                                         'content': self.toc.ncx_totalPageCount}))

        head.children.append(Element('meta', attributes={'name': 'dtb:maxPageNumber',
                                                         'content': self.toc.ncx_maxPageNumber}))

        head.children.append(Element('meta', attributes={'name': 'dtb:generator',
                                                         'content': 'epubaker ' + epubaker.version.__version__}))

        doc_title = Element('docTitle')
        ncx.children.append(doc_title)

        text = Element('text')
        doc_title.children.append(text)

        text.children.append(self.toc.title)

        nav_map = Element('navMap')
        ncx.children.append(nav_map)

        def make_nav_point(sec):
            nav_point = Element('navPoint', attributes={'id': 'id_' + uuid.uuid4().hex})

            nav_label = Element('navLabel')
            nav_point.children.append(nav_label)

            text_ = Element('text')
            nav_label.children.append(text_)

            text_.children.append(sec.title)

            content = Element('content')
            nav_point.children.append(content)

            first_sub = None
            for subsection in sec.subs:
                sub = make_nav_point(subsection)
                nav_point.children.append(sub)

                first_sub = first_sub or sub

            if sec.href:
                content.attributes[(None, 'src')] = sec.href

            elif first_sub:
                content.attributes[(None, 'src')] = first_sub.children[1].attributes[(None, 'src')]

                #
            return nav_point

        for _sec in self.toc:
            nav_map.children.append(make_nav_point(_sec))

        return ncx
        # return pretty_insert(ncx, dont_do_when_one_child=True).string()

    def _make_manifest_element(self):
        """
        :return: Manifest Element
         :rtype: Element
        """
        manifest = Element('manifest')

        ids = []

        item_dict = {}

        pathes = list(self.files.keys())

        def make_item(path, file_):
            item = Element('item', attributes={(None, 'href'): path})

            item.attributes[(None, 'media-type')] = file_.mime or mimes.map_from_extension[
                os.path.splitext(path)[1]]

            identification = xml_identify(path)
            new_id = identification
            i = 1
            while new_id in ids:
                new_id = identification + '_' + str(i)

            item.attributes[(None, 'id')] = new_id
            ids.append(new_id)

            return item

        while pathes:
            for _path in pathes:
                if self.files[_path].fallback is not None:
                    if self.files[_path].fallback in item_dict.keys():
                        _item = make_item(_path, self.files[_path])
                        _item.attributes[(None, 'fallback')] = item_dict[_path].attributes[(None, 'id')]
                        item_dict[_path] = _item

                        manifest.children.append(_item)
                        pathes.remove(_path)
                else:
                    _item = make_item(_path, self.files[_path])
                    item_dict[_path] = _item

                    manifest.children.append(_item)
                    pathes.remove(_path)

        for _path, _file in self._temp_files.items():
            manifest.children.append(make_item(_path, _file))

        return manifest

    def _make_spine_element(self):
        spine = Element('spine')

        for joint in self.spine:

            itemref = Element('itemref', attributes={(None, 'idref'): self._find_id(joint.path)})

            if joint.linear is True:
                itemref.attributes[(None, 'linear')] = 'yes'
            elif joint.linear is False:
                itemref.attributes[(None, 'linear')] = 'no'

            spine.children.append(itemref)

        return spine

    @staticmethod
    def _get_container_xmlstring(opf_path):
        e = Element('container')

        e.attributes['version'] = '1.0'

        e.prefixes['urn:oasis:names:tc:opendocument:xmlns:container'] = None

        rootfiles = Element('rootfiles')
        e.children.append(rootfiles)

        rootfile = Element('rootfile')
        rootfiles.children.append(rootfile)

        rootfile.attributes['full-path'] = opf_path

        rootfile.attributes['media-type'] = 'application/oebps-package+xml'

        return Xl(root=pretty_insert(e, dont_do_when_one_child=True)).string()

    def _get_unused_filename(self, dire, filename):
        dire = dire or ''

        only_name, ext = os.path.splitext(filename)
        unused_filename = filename

        while ROOT_OF_OPF + dire + '/' + unused_filename in [ROOT_OF_OPF + '/' + path for path in self.files.keys()] +\
                [ROOT_OF_OPF + '/' + path for path in self._temp_files.keys()]:

            unused_filename = '_{}{}'.format(
                random.random(''.join(random.sample(string.ascii_letters + string.digits, 8))),
                ext
            )

        return unused_filename

    @abstractmethod
    def write(self, filename):
        """Write to file.

        :param filename: file name.
        :type filename: str
        """

    ####################################################################################################################
    # Add-ons
    def addons_make_image_page(self, image_path, cover_page_path=None, width=None, heigth=None):
        """Make xhtml cover page contain the image you given.

        You must put the returned file to :class:`Epub2.files` by yourself

        :param image_path: Image path in your Epub2.files
        :param cover_page_path: Use this to get relative path to the image path
        :param width: Image width, automatic recognition if None
        :param heigth: Image heigth, automatic recognition if None
        :return: Cover xhtml page file.
        :rtype: File
        """

        img = Image.open(io.BytesIO(self.files[image_path].binary))
        width = width or img.size[0]
        height = heigth or img.size[1]

        relative = relative_path(os.path.split(cover_page_path or '')[0], image_path)

        xhtml_string = open(os.path.join(os.path.dirname(__file__), 'static', 'image_page.xhtml')).read()
        cover_page = xhtml_string.format(title='Cover', width=width, height=height, image_href=relative).encode()

        return File(cover_page)


def xml_identify(s):
    """
    :param s:
    :type s: str
    :return:
    :rtype: str
    """
    new_string = ''

    for char in s:
        if char.isalpha() or char.isdigit() or char in ('.', '_', '-'):
            new_string += char

        else:
            new_string += '__'

    if not new_string[0].isalpha():
        new_string = 'P_' + new_string

    return new_string



# coding=utf-8

from epubaker.xl import Element


class Meta2(object):
    """meta for Epub2.metadata"""
    def __init__(self, name, content):
        super(Meta2, self).__init__()
        self._name = name
        self._content = content

    @property
    def name(self):
        return self._name

    @property
    def content(self):
        return self._content

    def to_element(self):
        return Element('meta', attributes={'name': self.name, 'content': self.content})


class Cover(object):
    """cover for Epub2.metadata"""
    def __init__(self, filepath):
        self.filepath = filepath



# coding=utf-8

import magic

from epubaker.xl import parse


def relative_path(in_dir, to_file_path):
    """
    if you got file with path "text/cover.xhtml" and it links to "image/cover.png"

    :param in_dir: "text"
    :param to_file_path: "image/cover.png"
    :return: "../image/cover.png"
    """
    paths = to_file_path.split('/')
    dirs = in_dir.split('/')
    if dirs[-1] == '':
        dirs = dirs[0:-1]
    l = len(paths) if len(paths) >= len(dirs) else len(dirs)
    for i in range(l):
        if len(paths) == i or len(dirs) == i or paths[i] != dirs[i]:
            return '/'.join(['..'] * len(dirs[i:]) + list(paths[i:]))


def identify_mime(binary):
    """

    :param binary: bytes html
    :return: mime
    """
    mime = magic.from_buffer(binary, mime=True).decode()

    xhtml_uri = 'http://www.w3.org/1999/xhtml'

    if mime == mime.HTML:
        try:
            root = parse(binary.decode()).root
            if root.tag == (None, 'html') and root.prefixs[xhtml_uri] is None:
                return mime.XHTML
        except KeyError:
            return mime.HTML

    return mime


def w3c_utc_date(date_time=None):
    """

    :param date_time: instance of datetime, default is datetime.utcnow()
    :return: like 'CCYY-MM-DDThh:mm:ssZ'
    """
    from datetime import datetime

    if date_time is not None:
        if not isinstance(date_time, datetime):
            raise TypeError
    else:
        date_time = datetime.utcnow()

    return '{:0>4d}-{:0>2d}-{:0>2d}T{:0>2d}:{:0>2d}:{:0>2d}Z'.format(date_time.year,  date_time.month, date_time.day,
                                                                     date_time.hour, date_time.minute, date_time.second)



from setuptools import setup
from distutils.util import convert_path


NAME = "epubaker"

main_ns = {}
ver_path = convert_path('{}/version.py'.format(NAME))
with open(ver_path) as ver_file:
    exec(ver_file.read(), main_ns)

DESCRIPTION = 'A module to build EPUB 2 or 3 document.'


URL = 'https://github.com/meng89/' + NAME

CLASSIFIERS = [
    'Intended Audience :: Developers',
    'License :: OSI Approved :: MIT License',
    'Operating System :: OS Independent',
    'Programming Language :: Python :: 2',
    'Programming Language :: Python :: 2.7',
    'Programming Language :: Python :: 3',
    'Programming Language :: Python :: 3.3',
    'Programming Language :: Python :: 3.4',
    'Programming Language :: Python :: 3.5',
    'Programming Language :: Python :: Implementation :: CPython',
    'Programming Language :: Python :: Implementation :: PyPy',
    'Topic :: Software Development :: Libraries :: Python Modules',
]
with open('requirements.txt') as f:
    requirements = f.read().splitlines()

setup(name=NAME,
      version=main_ns['__version__'],
      description=DESCRIPTION,
      include_package_data=True,
      author='Chen Meng',
      author_email='ObserverChan@gmail.com',
      license='MIT',
      url=URL,
      packages=[
          'epubaker',
          'epubaker.metas',
          'epubaker.xl'
      ],
      install_requires=requirements,
      classifiers=CLASSIFIERS)



# coding=utf-8

from .dcmes import Identifier, Language, Title
from .dcmes import Contributor, Coverage, Creator, Date, Description, Format, Publisher, Relation, Rights
from .dcmes import Source, Subject, Type

from .dcterms import get_dcterm
from .epub3_meta import Meta3

from .epub2_meta import Cover, Meta2



# coding=utf-8

from __future__ import unicode_literals

import zipfile

import html5lib
import io
import os


from epubaker.epub import Epub, File, OPF_NS, ROOT_OF_OPF, CONTAINER_PATH

from epubaker.metas.dcmes import URI_DC

from epubaker.xl import Xl, Element, URI_XML, pretty_insert

from epubaker import mimes


XML_URI = 'http://www.w3.org/1999/xhtml'
OPS_URI = 'http://www.idpf.org/2007/ops'


class Epub3(Epub):
    def __init__(self):
        Epub.__init__(self)

        self._cover_image = None

    @property
    def cover_image(self):
        """Tag your cover image path as a cover"""
        return self._cover_image

    @cover_image.setter
    def cover_image(self, path):
        if path is not None and path not in self.files.keys():
            raise ValueError()

        self._cover_image = path

    def _make_metadata_element(self):
        """
        :return: Metadata element
        :rtype: Element
        """
        metadata = Element('metadata', prefixes={URI_DC: 'dc'})
        for m in self.metadata:
            metadata.children.append(m.to_element())

        return metadata

    def _make_nav_element(self):
        """
        :return: Nav html element
        :rtype: Element
        """

        html = Element((None, 'html'), prefixes={XML_URI: None, OPS_URI: 'epub'})

        head = Element((None, 'head'))
        html.children.append(head)

        if self.toc.title:
            _title = Element((None, 'title'))
            head.children.append(_title)
            _title.children.append(self.toc.title)

        body = Element((None, 'body'))
        html.children.append(body)

        def to_nav_li_element(sec):
            li = Element((None, 'li'))

            if sec.href:
                a_or_span = Element((None, 'a'))
                a_or_span.attributes[(None, 'href')] = sec.href
            else:
                a_or_span = Element((None, 'span'))

            a_or_span.children.append(sec.title)

            li.children.append(a_or_span)

            if sec.subs:
                _ol = Element((None, 'ol'))

                if sec.hidden_subs:
                    _ol.attributes[(None, 'hidden')] = ''

                for one in sec.subs:
                    _ol.children.append(to_nav_li_element(one))

                li.children.append(_ol)

            return li

        if self.toc:
            nav = Element((None, 'nav'), prefixes={OPS_URI: 'epub'}, attributes={(OPS_URI, 'type'): 'toc'})
            ol = Element((None, 'ol'))

            for section in self.toc:
                ol.children.append(to_nav_li_element(section))

            nav.children.append(ol)
            body.children.append(nav)

        return html

    def _process_items_properties(self, manifest):
        for item in manifest.children:
            properties = []

            if item.attributes[(None, 'media-type')] in (mimes.XHTML, mimes.HTML):

                try:
                    html_string = self.files[item.attributes[(None, 'href')]].binary.decode()
                except KeyError:
                    html_string = self._temp_files[item.attributes[(None, 'href')]].binary.decode()

                if _has_element('script', html_string):
                    properties.append('scripted')

                if _has_element('math', html_string):
                    properties.append('mathml')

            if properties:
                item.attributes[(None, 'properties')] = ' '.join(properties)

    def _get_opf_xmlstring(self, toc_path):

        package = Element('package', prefixes={OPF_NS: None}, attributes={'version': '3.0'})

        package.attributes[(URI_XML, 'lang')] = 'en'

        # unique - identifier = "pub-id"
        if self._find_unique_id():
            package.attributes['unique-identifier'] = self._find_unique_id()

        # Metadata
        package.children.append(self._make_metadata_element())

        # manifest
        manifest = self._make_manifest_element()
        package.children.append(manifest)
        self._process_items_properties(manifest)

        for item in manifest.children:
            if item.attributes[(None, 'href')] == toc_path:
                item.attributes[(None, 'properties')] = 'nav'

            if item.attributes[(None, 'href')] == self.cover_image:
                item.attributes[(None, 'properties')] = 'cover-image'

        # Find ncx id for spine
        toc_ncx_item_e_id = self._find_ncx_id(manifest.children)

        # Spine
        spine = self._make_spine_element()
        package.children.append(spine)
        spine.attributes['toc'] = toc_ncx_item_e_id

        return Xl(root=pretty_insert(package, dont_do_when_one_child=True)).string()

    def write(self, filename):

        # put nav to temp files
        nav_xmlstring = pretty_insert(self._make_nav_element(), dont_do_when_one_child=True).string()
        toc_nav_path = self._get_unused_filename(None, 'nav.xhtml')
        self._temp_files[toc_nav_path] = File(nav_xmlstring.encode(), mime='application/xhtml+xml')

        # put ncx to temp files
        ncx_xmlstring = pretty_insert(self._make_ncx_element(), dont_do_when_one_child=True).string()
        toc_ncx_filename = self._get_unused_filename(None, 'toc.ncx')
        self._temp_files[toc_ncx_filename] = File(ncx_xmlstring.encode(), mime='application/x-dtbncx+xml')

        # get opf name & data
        opf_data = self._get_opf_xmlstring(toc_nav_path).encode()
        opf_filename = self._get_unused_filename(None, 'package.opf')

        # get container data
        container_data = self._get_container_xmlstring(ROOT_OF_OPF + '/' + opf_filename).encode()

        # make zipfile
        z = zipfile.ZipFile(filename, 'w', compression=zipfile.ZIP_DEFLATED)

        # write mimetype as first file in zip
        z.writestr('mimetype', 'application/epub+zip'.encode('ascii'), compress_type=zipfile.ZIP_STORED)

        # wirte files
        for filename, _file in self.files.items():
            z.writestr(ROOT_OF_OPF + os.sep + filename, _file.binary, zipfile.ZIP_DEFLATED)

        # write temp files
        for filename, _file in self._temp_files.items():
            z.writestr(ROOT_OF_OPF + os.sep + filename, _file.binary, zipfile.ZIP_DEFLATED)

        # write opf data
        z.writestr(ROOT_OF_OPF + '/' + opf_filename, opf_data, zipfile.ZIP_DEFLATED)

        # write container
        z.writestr(CONTAINER_PATH, container_data, zipfile.ZIP_DEFLATED)

        z.close()

        self._temp_files.clear()

    write.__doc__ = Epub.write.__doc__

    ####################################################################################################################
    # Add-ons
    def addons_make_toc_page(self):
        """Some EPUB reader not supports nav hidden attribute, they just ignor sub section, not fold.
        So, this member function can make a toc page, with it's little JS code, it can fold and unfold sections.

        You must put the returned file to Epub3.files by yourself.

        :returns: xhtml page file
        :rtype: File
        """

        html = self._make_nav_element()

        def find_element_by_name(tag):
            e = None
            for one in html.children:
                if one.tag == tag:
                    return one

            return e

        head = find_element_by_name((None, 'head'))
        body = find_element_by_name((None, 'body'))

        css_string = open(os.path.join(_dirt(__file__), 'static', 'user_toc_nav.css')).read()
        css = Element('style', attributes={'type': 'text/css'})
        css.children.append(css_string)

        head.children.append(css)

        js_string = io.open(os.path.join(_dirt(__file__), 'static', 'user_toc_nav_fold.js'), encoding='utf-8').read()
        script = Element('script')
        script.children.append(js_string)

        head.children.append(script)

        script_before_body_close = Element('script', attributes={'type': 'text/javascript'})
        script_before_body_close.children.append('set_button();')
        body.children.append(script_before_body_close)

        toc_page = pretty_insert(html).string().encode()
        return File(toc_page)


def _has_element(tag, file_string):
    parser = html5lib.HTMLParser(tree=html5lib.getTreeBuilder('dom'))
    minidom_docment = parser.parse(file_string)

    if minidom_docment.getElementsByTagName(tag):
        return True
    else:
        return False


def _dirt(_file):
    return os.path.dirname(_file)



# coding=utf-8

from __future__ import print_function

from xml.etree import ElementTree as Et

from epubaker.xl import parse, clean_whitespaces, pretty_insert

import os

path = os.path.dirname(__file__)

TEST_XML_PATH = os.path.join(path, 'test.xml')


def test_parse():
    xmlstr = open(TEST_XML_PATH).read()
    xl = parse(xmlstr)

    e = xl.root

    clean_e = clean_whitespaces(e)

    pretty_e = pretty_insert(clean_e, dont_do_when_one_child=True)

    assert clean_e.string() == clean_whitespaces(pretty_e).string()

    assert pretty_e.string() == pretty_insert(clean_e, dont_do_when_one_child=True).string()

    assert clean_e.string() != pretty_e.string()

    Et.fromstring(xl.string())

    xl.root = clean_e
    Et.fromstring(xl.string())

    xl.root = pretty_e
    Et.fromstring(xl.string())



import uuid
import zipfile
from xml.etree import ElementTree as Et

import os

from epubaker.metas import Title, Language, Identifier

from epubaker import Joint, File, Section

XHTML_TEMPLATE = """
<html xmlns="http://www.w3.org/1999/xhtml">
<head>
<title>{title}</title>
</head>
<body><p>{content}</p></body></html>
"""

cur_path = os.path.dirname(__file__)

BUILT_BOOK_DIR = '_test_built_book'
if not os.path.exists(BUILT_BOOK_DIR):
    os.makedirs(BUILT_BOOK_DIR)


def make_epub(epub, section):
    book = epub()

    # metadata
    book.metadata.append(Title('EPUB demo'))
    book.metadata.append(Language('en'))
    book.metadata.append(Identifier('identifier_' + uuid.uuid4().hex))

    def make_page(title, html_path=None, content=None):
        if html_path and content:
            _file = File(XHTML_TEMPLATE.format(title=title, content=content).encode(), mime='application/xhtml+xml')
            book.files[html_path] = _file

            book.spine.append(Joint(html_path))

        sec = section(title, href=html_path)
        return sec

    sec1 = make_page('Part I', 'Part_I.xhtml',  content='This is Part I content')
    sec1_1 = make_page('Chapter 1', 'pi_c1.xhtml', content='This is Chapter 1 content')
    sec1_2 = make_page('Chapter 2', 'pi_c2.xhtml', content='This is Chapter 2 content')
    sec1.hidden_subs = True
    sec1.subs.extend([sec1_1, sec1_2])

    sec2 = make_page('Part II')
    sec2_1 = make_page('Chapter 1', 'pii_c1.xhtml', content='This is Chapter 1 content')
    sec2_2 = make_page('Chapter 2', 'pii_c2.xhtml', content='This is Chapter 2 content')
    sec2.subs.extend([sec2_1, sec2_2])

    sec3 = make_page('Part III', 'Part_III.xhtml', content='This is Part III content')
    sec3_1 = make_page('Chapter 1', 'piii_c1.xhtml', content='This is Chapter 1 content')
    sec3_2 = make_page('Chapter 2', 'piii_c2.xhtml', content='This is Chapter 2 content')
    sec3.subs.extend([sec3_1, sec3_2])

    book.toc.extend([sec1, sec2, sec3])

    return book


def check_xml(book_path):
    z = zipfile.ZipFile(book_path, 'r')
    container = Et.fromstring(z.read('META-INF/container.xml').decode())
    for rootfile in container[0]:
        if rootfile.attrib['media-type'] == 'application/oebps-package+xml':
            Et.fromstring(z.read(rootfile.attrib['full-path']))


def test_epub3():
    from epubaker import Epub3
    from epubaker.metas import get_dcterm
    from epubaker.tools import w3c_utc_date

    book = make_epub(Epub3, Section)

    book.metadata.append(get_dcterm('modified')(w3c_utc_date()))

    book.files['cover.png'] = File(open(os.path.join(cur_path, 'cover', 'cover.png'), 'rb').read())
    book.cover_image = 'cover.png'

    # make user toc
    toc_page_path = 'user_toc.xhtml'
    toc_page_file = book.addons_make_toc_page()
    book.files[toc_page_path] = toc_page_file
    book.spine.insert(0, Joint(toc_page_path))
    book.toc.insert(0, Section('Table of Contents', href=toc_page_path))

    book_path = os.path.join(BUILT_BOOK_DIR, '3.epub')
    book.write(book_path)

    check_xml(book_path)


def test_epub2():
    from epubaker import Epub2
    from epubaker.metas import Cover

    book = make_epub(Epub2, Section)

    cover_img_file = File(open(os.path.join(cur_path, 'cover', 'cover.png'), 'rb').read())
    cover_img_path = 'cover.png'
    book.files['cover.png'] = cover_img_file
    book.metadata.append(Cover(cover_img_path))

    cover_page_path = 'cover.xhtml'
    cover_page_file = book.addons_make_image_page(image_path='cover.png', cover_page_path=cover_page_path)
    book.files[cover_page_path] = cover_page_file
    book.spine.insert(0, Joint(cover_page_path))

    book.toc.ncx_depth = 1

    book_path = os.path.join(BUILT_BOOK_DIR, '2.epub')
    book.write(book_path)

    check_xml(book_path)
